@@ -1,6 +1,6 @@
 /*
     WRK File component
-    Copyright (C) 2010-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2010-2023, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <QObject>
 #include <QScopedPointer>
 
+class QTextCodec;
 class QDataStream;
 
 /**
@@ -30,6 +31,16 @@ class QDataStream;
  * Cakewalk WRK Files Input
  */
 
+#if defined(DRUMSTICK_STATIC)
+#define DRUMSTICK_FILE_EXPORT
+#else
+#if defined(drumstick_file_EXPORTS)
+#define DRUMSTICK_FILE_EXPORT Q_DECL_EXPORT
+#else
+#define DRUMSTICK_FILE_EXPORT Q_DECL_IMPORT
+#endif
+#endif
+
 namespace drumstick { namespace File {
 
 /**
@@ -58,6 +69,7 @@ enum WrkChunkType {
     LYRICS_CHUNK = 18,   ///< Events stream with lyrics
     TRKVOL_CHUNK = 19,   ///< Track volume
     SYSEX2_CHUNK = 20,   ///< System exclusive bank
+    MARKERS_CHUNK = 21,  ///< Markers
     STRTAB_CHUNK = 22,   ///< Table of text event types
     METERKEY_CHUNK = 23, ///< Meter/Key map
     TRKNAME_CHUNK = 24,  ///< Track name
@@ -75,10 +87,12 @@ enum WrkChunkType {
 /**
  * Cakewalk WRK file format (input only)
  *
- * This class is used to parse Cakewalk WRK Files
+ * This class is used to parse Cakewalk WRK Files.
+ * Signals with QString parameters are deprecated because the class QTextCodec was removed from QtCore since Qt6.
+ *
  * @since 0.3.0
  */
-class DRUMSTICK_EXPORT QWrk : public QObject
+class DRUMSTICK_FILE_EXPORT QWrk : public QObject
 {
     Q_OBJECT
     Q_ENUM(WrkChunkType)
@@ -88,8 +102,8 @@ public:
 
     void readFromStream(QDataStream *stream);
     void readFromFile(const QString& fileName);
-    QTextCodec* getTextCodec();
-    void setTextCodec(QTextCodec *codec);
+    Q_DECL_DEPRECATED QTextCodec* getTextCodec();
+    Q_DECL_DEPRECATED void setTextCodec(QTextCodec *codec);
     long getFilePos();
 
     int getNow() const;
@@ -257,8 +271,10 @@ Q_SIGNALS:
      * @param time musical time
      * @param type Text type
      * @param data Text data
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKText2() instead
      */
-    void signalWRKText(int track, long time, int type, const QString& data);
+    Q_DECL_DEPRECATED void signalWRKText(int track, long time, int type, const QString& data);
 
     /**
      * Emitted after reading a WRK Time signature
@@ -301,7 +317,10 @@ Q_SIGNALS:
      * @param selected true if track is selected
      * @param muted true if track is muted
      * @param loop true if loop is enabled
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKTrack2() instead
      */
+    Q_DECL_DEPRECATED
     void signalWRKTrack(const QString& name1,
                         const QString& name2,
                         int trackno, int channel, int pitch,
@@ -385,8 +404,10 @@ Q_SIGNALS:
      * Emitted after reading a comments chunk
      *
      * @param data file text comments
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKComments2() instead
      */
-    void signalWRKComments(const QString& data);
+    Q_DECL_DEPRECATED void signalWRKComments(const QString& data);
 
     /**
      * Emitted after reading a variable chunk.
@@ -417,7 +438,10 @@ Q_SIGNALS:
      * @param selected track is selected
      * @param muted track is muted
      * @param loop track loop enabled
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKNewTrack2() instead
      */
+    Q_DECL_DEPRECATED
     void signalWRKNewTrack( const QString& name,
                             int trackno, int channel, int pitch,
                             int velocity, int port,
@@ -435,15 +459,19 @@ Q_SIGNALS:
      *
      * @param track track number
      * @param name track name
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKTrackName2() instead
      */
-    void signalWRKTrackName(int track, const QString& name);
+    Q_DECL_DEPRECATED void signalWRKTrackName(int track, const QString& name);
 
     /**
      * Emitted after reading a string event types chunk.
      *
      * @param strs list of declared string event types
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKStringTable2() instead
      */
-    void signalWRKStringTable(const QStringList& strs);
+    Q_DECL_DEPRECATED void signalWRKStringTable(const QStringList& strs);
 
     /**
      * Emitted after reading a segment prefix chunk.
@@ -451,8 +479,10 @@ Q_SIGNALS:
      * @param track track number
      * @param time segment time offset
      * @param name segment name
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKSegment2() instead
      */
-    void signalWRKSegment(int track, long time, const QString& name);
+    Q_DECL_DEPRECATED void signalWRKSegment(int track, long time, const QString& name);
 
     /**
      * Emitted after reading a chord diagram chunk.
@@ -471,8 +501,10 @@ Q_SIGNALS:
      * @param time event time in ticks
      * @param code expression event code
      * @param text expression text
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKExpression2() instead
      */
-    void signalWRKExpression(int track, long time, int code, const QString& text);
+    Q_DECL_DEPRECATED void signalWRKExpression(int track, long time, int code, const QString& text);
 
     /**
      * Emitted after reading a hairpin symbol (notation) chunk.
@@ -484,6 +516,125 @@ Q_SIGNALS:
      */
     void signalWRKHairpin(int track, long time, int code, int dur);
 
+    /**
+     * Emitted after reading a text message
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param track track number
+     * @param time musical time
+     * @param type Text type
+     * @param data Text data
+     */
+    void signalWRKText2(int track, long time, int type, const QByteArray& data);
+
+    /**
+     * Emitted after reading a track prefix chunk
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param name1 track 1st name
+     * @param name2 track 2nd name
+     * @param trackno track number
+     * @param channel track forced channel (-1=no forced)
+     * @param pitch track pitch transpose in semitones (-127..127)
+     * @param velocity track velocity increment (-127..127)
+     * @param port track forced port
+     * @param selected true if track is selected
+     * @param muted true if track is muted
+     * @param loop true if loop is enabled
+     */
+    void signalWRKTrack2(const QByteArray& name1,
+                        const QByteArray& name2,
+                        int trackno, int channel, int pitch,
+                        int velocity, int port,
+                        bool selected, bool muted, bool loop );
+
+    /**
+     * Emitted after reading a comments chunk
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param data file text comments
+     */
+    void signalWRKComments2(const QByteArray& data);
+
+    /**
+     * Emitted after reading a new track prefix
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param name track name
+     * @param trackno track number
+     * @param channel forced MIDI channel
+     * @param pitch Note transposition
+     * @param velocity Velocity increment
+     * @param port MIDI port number
+     * @param selected track is selected
+     * @param muted track is muted
+     * @param loop track loop enabled
+     */
+    void signalWRKNewTrack2(const QByteArray& name,
+                            int trackno, int channel, int pitch,
+                            int velocity, int port,
+                            bool selected, bool muted, bool loop );
+    /**
+     * Emitted after reading a track name chunk.
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param track track number
+     * @param name track name
+     */
+    void signalWRKTrackName2(int track, const QByteArray& name);
+
+    /**
+     * Emitted after reading a string event types chunk.
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param strs list of declared string event types
+     */
+    void signalWRKStringTable2(const QList<QByteArray>& strs);
+
+    /**
+     * Emitted after reading a segment prefix chunk.
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param track track number
+     * @param time segment time offset
+     * @param name segment name
+     */
+    void signalWRKSegment2(int track, long time, const QByteArray& name);
+
+    /**
+     * Emitted after reading an expression indication (notation) chunk.
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param track track number
+     * @param time event time in ticks
+     * @param code expression event code
+     * @param text expression text
+     */
+    void signalWRKExpression2(int track, long time, int code, const QByteArray& text);
+
+    /**
+     * Emitted after reading a text marker
+     * This is deprecated because the class QTextCodec was removed from QtCore since Qt6
+     * Use signalWRKMarker2() instead
+     *
+     * @param time event time in ticks or smpte
+     * @param type tipe of time: 0=ticks or 1=smpte
+     * @param data marker text
+     * @deprecated because the class QTextCodec was removed from QtCore since Qt6.
+     * use signalWRKMarker2() instead
+     */
+    Q_DECL_DEPRECATED void signalWRKMarker(long time, int type, const QString& data);
+
+    /**
+     * Emitted after reading a text marker
+     * This signal is emitted when getTextCodec() is nullptr
+     *
+     * @param time event time in ticks or smpte
+     * @param type tipe of time: 0=ticks or 1=smpte
+     * @param data marker text
+     */
+    void signalWRKMarker2(long time, int type, const QByteArray& data);
+
 private:
     quint8 readByte();
     quint16 to16bit(quint8 c1, quint8 c2);
@@ -530,6 +681,9 @@ private:
     void processUnknown(int id);
     void processEndChunk();
     void wrkRead();
+    QByteArray readByteArray(int len);
+    QByteArray readVarByteArray();
+    void processMarkers();
 
     struct RecTempo {
         long time;
